@@ -13,6 +13,7 @@ import os
 import re
 import sys
 import urllib
+import urllib2
 
 import mozinfo
 
@@ -50,7 +51,8 @@ class Scraper(object):
     """Generic class to download an application from the Mozilla server"""
 
     def __init__(self, directory, version, platform=None,
-                 application='firefox', locale='en-US', extension=None):
+                 application='firefox', locale='en-US', extension=None,
+                 authentication=None):
 
         # Private properties for caching
         self._target = None
@@ -61,6 +63,7 @@ class Scraper(object):
         self.platform = platform or self.detect_platform()
         self.version = version
         self.extension = extension or DEFAULT_FILE_EXTENSIONS[self.platform]
+        self.authentication = authentication
 
         # build the base URL
         self.application = application
@@ -163,18 +166,35 @@ class Scraper(object):
             os.makedirs(self.directory)
 
         try:
-            # Don't re-download the build
+            # Don't re-download the file
             if os.path.isfile(os.path.abspath(self.target)):
-                print "Build has already been downloaded: %s" % (self.target)
+                print "File has already been downloaded: %s" % (self.target)
                 return
 
-            print 'Downloading build: %s' % (urllib.unquote(self.final_url))
+            print 'Downloading from: %s' % (urllib.unquote(self.final_url))
             tmp_file = self.target + ".part"
-            urllib.urlretrieve(self.final_url, tmp_file)
+
+            if self.authentication \
+               and self.authentication['username'] \
+               and self.authentication['password']:
+                password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
+                password_mgr.add_password(None,
+                                          self.final_url,
+                                          self.authentication['username'],
+                                          self.authentication['password'])
+                handler = urllib2.HTTPBasicAuthHandler(password_mgr)
+                opener = urllib2.build_opener(urllib2.HTTPHandler, handler)
+                urllib2.install_opener(opener)
+
+            r = urllib2.urlopen(self.final_url)
+            CHUNK = 16 * 1024
+            with open(tmp_file, 'wb') as f:
+                for chunk in iter(lambda: r.read(CHUNK), ''):
+                    f.write(chunk)
             os.rename(tmp_file, self.target)
         except:
             try:
-                if tmp_file:
+                if os.path.isfile(tmp_file):
                     os.remove(tmp_file)
             except OSError:
                 pass
@@ -315,6 +335,23 @@ class DailyScraper(Scraper):
                                     self.base_url + self.monthly_build_list_regex)
 
 
+class DirectScraper(Scraper):
+    """Class to download a file from a specified URL"""
+
+    def __init__(self, url, *args, **kwargs):
+        Scraper.__init__(self, *args, **kwargs)
+
+        self.url = url
+
+    @property
+    def target(self):
+        return urllib.splitquery(self.final_url)[0].rpartition('/')[-1]
+
+    @property
+    def final_url(self):
+        return self.url
+
+
 class ReleaseScraper(Scraper):
     """Class to download a release build from the Mozilla server"""
 
@@ -500,8 +537,8 @@ class TinderboxScraper(Scraper):
                         'linux64': r'.*\.%(EXT)s$',
                         'mac': r'.*\.%(EXT)s$',
                         'mac64': r'.*\.%(EXT)s$',
-                        'win32': r'.*\.%(EXT)s$',
-                        'win64': r'.*\.%(EXT)s$'}
+                        'win32': r'.*(\.installer)\.%(EXT)s$',
+                        'win64': r'.*(\.installer)\.%(EXT)s$'}
 
         regex = regex_base_name + regex_suffix[self.platform]
 
@@ -661,6 +698,11 @@ def cli():
                       default='release',
                       metavar='BUILD_TYPE',
                       help='Type of build to download, default: "%default"')
+    parser.add_option('--url',
+                      dest='url',
+                      default=None,
+                      metavar='URL',
+                      help='URL to download.')
     parser.add_option('--version', '-v',
                       dest='version',
                       metavar='VERSION',
@@ -672,6 +714,16 @@ def cli():
                       metavar='EXTENSION',
                       help='File extension of the build (e.g. "zip"), default:\
                             the standard build extension on the platform.')
+    parser.add_option('--username',
+                      dest='username',
+                      default=None,
+                      metavar='USERNAME',
+                      help='Username for basic HTTP authentication.')
+    parser.add_option('--password',
+                      dest='password',
+                      default=None,
+                      metavar='PASSWORD',
+                      help='Password for basic HTTP authentication.')
 
     # Option group for candidate builds
     group = OptionGroup(parser, "Candidate builds",
@@ -717,7 +769,8 @@ def cli():
 
     # Check for required options and arguments
     # Note: Will be optional when ini file support has been landed
-    if not options.type in ['daily', 'tinderbox'] \
+    if not options.url \
+       and not options.type in ['daily', 'tinderbox'] \
        and not options.version:
         parser.error('The version of the application to download has not been specified.')
 
@@ -727,7 +780,11 @@ def cli():
                         'platform': options.platform,
                         'version': options.version,
                         'directory': options.directory,
-                        'extension': options.extension}
+                        'extension': options.extension,
+                        'authentication': {
+                            'username': options.username,
+                            'password': options.password}
+                        }
     scraper_options = {'candidate': {
                            'build_number': options.build_number,
                            'no_unsigned': options.no_unsigned},
@@ -745,7 +802,11 @@ def cli():
 
     kwargs = scraper_keywords.copy()
     kwargs.update(scraper_options.get(options.type, {}))
-    build = BUILD_TYPES[options.type](**kwargs)
+
+    if options.url:
+        build = DirectScraper(options.url, **kwargs)
+    else:
+        build = BUILD_TYPES[options.type](**kwargs)
 
     build.download()
 
